-- VARIABLES GLOBAL TO ADDON

local nbNameFormatted = "|cFF88FF44NoseBag|r"
local nbGray = "|cFF777777"
local nbRed = "|cFFFF0000"
local nbYellow = "|cFFFFFF00"
local nbGreen = "|cFF00FF00"
local nbWhite = "|cFFFFFFFF"
local nbLoaded = false					-- Set at the end of initialization. Prevents multiple inits.
local nbInCombat = false				-- Set when player enters combat. Reset when combat ends.

	-- PLAYER_LOGIN
	
local nbPlayerLevel = 1					-- Tracks player level. Used to avoid unnecessary calls to UnitLevel.

	-- Pet
	
local nbAlertTicksRemaining = 0			-- Remaining calls to nbCheckPetHappiness before player should be alerted.
local nbAlertDelayTicks = 7				-- Number of calls to nbCheckPetHappiness between alerts.
local nbHappinessCheckSeconds = 1		-- Time between calls to nbCheckPetHappiness.

	-- Button
local nbButtonIsVisible = true			-- Tracks button visibility to avoid unnecessary calls to Show()
local nbBtn = CreateFrame("Button",nil,UIParent,"SecureActionButtonTemplate")	-- Main button. Other setup in Initialize.
nbBtn:SetNormalFontObject(GameFontNormalHuge)
local f = nbBtn:GetNormalFontObject()
f:SetTextColor(1.0,1.0,1.0,1.0)
nbBtn:SetNormalFontObject(f)

nbBtn:RegisterEvent("PLAYER_LOGIN")			-- Used to kick off initialization process.
nbBtn:RegisterEvent("PLAYER_REGEN_DISABLED")-- Fires at start of combat.
nbBtn:RegisterEvent("PLAYER_REGEN_ENABLED")	-- Fires at end of combat.

	-- Button State Data & Functions
	
local function nbShowButton()				-- Wrapper for Show to avoid changing button during combat.
	if not nbInCombat then
		nbButtonIsVisible = true
		nbBtn:Show()
	end
end

local function nbHideButton()				-- Wrapper for Hide to avoid changing button during combat.
		nbButtonIsVisible = false
	if not nbInCombat then
		nbBtn:Hide()
	end
end

	-- Note: Multiple interactions with the button during combat will be overwritten by the most recent one.
	-- 			For example, if the button is cleared and then a new food is dragged onto it while combat is
	--			underway, only the new food will be applied to the button.
local nbPendingTexture = ""					-- Texture to be applied to button after combat if changed during combat.
local nbPendingMacro = ""					-- Macro to ba applied to button after combat if changed during combat.

-- If combat is underway then delay 1 second and retry. Otherwise, apply new
-- attributes to the button.
local function nbApplyPendingButtonAttributes()
	if nbInCombat then
		C_Timer.After(1, nbApplyPendingButtonAttributes)
	else
		nbBtn:SetNormalTexture(nbPendingTexture)
		nbBtn:SetAttribute("macrotext", nbPendingMacro)
	end
end

-- Set changes to button attributes to be applied as soon as possible.
local function nbSetPendingButtonChanges(texture, macro)
	nbPendingTexture = texture
	nbPendingMacro = macro
	nbApplyPendingButtonAttributes()
end

-- Update the button based on currently set food.
local function nbUpdateButtonState(forceButtonUpdate)
	local state, count
	
	if svActiveFood == nil then
		state = "EMPTY"
		nbBtn:SetText("")
	else
		count = GetItemCount(svActiveFood)
		if count < 1 then
			state = "EMPTY"
			nbBtn:SetText("")
		else
			state = "FEED"
			nbBtn:SetText(count)
		end
	end
	
	if state ~= svButtonState or forceButtonUpdate then
		svButtonState = state
		if svButtonState == "FEED" then
			local foodName = GetItemInfo(svActiveFood)
			local icon = GetItemIcon(svActiveFood)
			nbSetPendingButtonChanges(icon, "/cast Feed Pet\n/use " .. foodName .. "\n/script nbNotifyButtonClicked()\n")
		elseif svButtonState == "EMPTY" then
			nbSetPendingButtonChanges("Interface\\ICONS\\INV_drink_17.blp", "/script OpenAllBags()")
		end
	end
end

-- Called from the button macro when feeding pet to notify us to update the button.
function nbNotifyButtonClicked()
	nbUpdateButtonState(true)
end
	
-- SOUND

-- Plays a sound if the silent flag isn't set.
local function nbPlaySound(sound, channel)
	if not svSilent then
		PlaySoundFile(sound, channel)
	end
end

-- TOOLTIP

-- Sets up the button tooltip based on its state.
local function nbUpdateTooltip()
	GameTooltip:SetOwner(nbBtn, "ANCHOR_TOPRIGHT")
	GameTooltip:ClearLines()
	local soundStatus
	if svSilent then
		soundStatus = "SHIFT-RIGHT-CLICK to turn sound on"
	else
		soundStatus = "SHIFT-RIGHT-CLICK to turn sound off"
	end
	if svButtonState == "EMPTY" then
		GameTooltip:AddLine(nbNameFormatted)
		GameTooltip:AddLine("DRAG and DROP pet food here.\nThen CLICK to feed.")
	elseif svButtonState == "FEED" then
		if svActiveFood then
			local itemName,_,_,itemLevel = GetItemInfo(svActiveFood)
			local levelDifference = UnitLevel("pet") - itemLevel
			local levelColor = nbGreen
			if levelDifference > 30 then
				levelColor = nbGray
			elseif levelDifference > 20 then
				levelColor = nbRed
			elseif levelDifference > 10 then
				levelColor = nbYellow
			else
				levelColor = nbGreen
			end
			GameTooltip:AddLine(levelColor .. itemName .. "|r")
			GameTooltip:AddDoubleLine(nbWhite .. "Level: " .. itemLevel, nbWhite .. "Available: " .. GetItemCount(svActiveFood))
			GameTooltip:AddLine("RIGHT-CLICK to clear")
		end
	end
	GameTooltip:AddLine(soundStatus)
end

-- PET

-- Alerts player that pet wants to be fed.
local function nbAlertFeed()
	nbAlertTicksRemaining = nbAlertTicksRemaining + 1
	if nbAlertTicksRemaining >= nbAlertDelayTicks then
		nbBtn:LockHighlight()
		if not nbInCombat then
			nbPlaySound("interface\\addons\\nosebag\\assets\\bark.mp3", "SFX")
		end
		nbAlertTicksRemaining = 0
	else
		nbBtn:UnlockHighlight()
	end
end

-- Check the pet for a Feed Pet Effect buff. Note that, while it
-- should be possible to pass "Feed Pet Effect" to UnitBuff, in
-- practice this appears not to be true. So, instead, we scan through
-- all buffs currently on the pet until we find it or run out of
-- buffs to check. If we find the buff, we return both a boolean and
-- the remaining time on the buff. We do it this way mainly for
-- convenince in places where we just want to do a simple test for the
-- buff's presence.
local function nbHasFeedEffect()
	local buffIndex = 1
	local buff,_,_,_,_,expirationTime = UnitBuff("pet", 1)
	while buff do
		if buff == "Feed Pet Effect" then
			return true, expirationTime - GetTime()
		end
		buffIndex = buffIndex + 1
		buff,_,_,_,_,expirationTime = UnitBuff("pet", buffIndex)
	end
	return false, 0
end

-- Called periodically to check on pet's happiness. Also checks player
-- level to see if button needs to be unhidden because player leveled.
local function nbCheckPetHappiness()
	-- Bail out if no chance of having a pet out.
	if UnitOnTaxi("player") then return end
	if InCinematic() then return end
	if IsFlying() then return end
	if IsStealthed() then return end
	
	if nbPlayerLevel < 10 then
		nbPlayerLevel = UnitLevel("player")
		if nbPlayerLevel > 9 then
			if not nbButtonIsVisible then
				nbShowButton()
			end
		end
	end
	
	-- Make sure we have a pet present.
	if HasPetUI() then
		local happiness = GetPetHappiness()
		local feeding = nbHasFeedEffect()
		if happiness < 3 and not feeding then
			nbAlertFeed()
		end
	end
end

-- INITIALIZE

-- Set up saved variables and configure the button. Also set up button event handlers.
local function nbInitialize()
	if not nbLoaded then
	
		-- SAVED VARIABLES AND BUTTON SETUP
		
		if not svXpos then
			svXpos = 0
		end
		if not svYpos then
			svYpos = 540
		end
		if not svRelativePoint then
			svRelativePoint = "BOTTOM"
		end
		nbBtn:SetAttribute("type1", "macro")
		nbBtn:SetWidth(64)
		nbBtn:SetHeight(64)
		nbBtn:SetClampedToScreen(true)
		nbBtn:SetHighlightTexture("Interface\\BUTTONS\\ButtonHilight-Square.blp")
		nbBtn:SetMovable(true)
		nbBtn:EnableMouse(true)
		nbBtn:RegisterForDrag("LeftButton")
		nbBtn:SetPoint(svRelativePoint,svXpos,svYpos)

		if not svButtonState then
			svActiveFood = nil
			svButtonState = "EMPTY"
		end
		nbUpdateButtonState(true)
		
		-- BUTTON DRAG
		
		nbBtn:SetScript("OnDragStart", function()
			nbBtn:StartMoving()
		end)
		
		nbBtn:SetScript("OnDragStop", function()
			nbBtn:StopMovingOrSizing()
			_,_,svRelativePoint,svXpos,svYpos = nbBtn:GetPoint()
		end)
		
		nbBtn:SetScript("OnReceiveDrag", function()
			local infoType, itemID = GetCursorInfo()
			if infoType == "item" then
				svActiveFood = itemID
				nbUpdateButtonState(true)
				nbUpdateTooltip()
				local _,_,_,petClass = GetStablePetInfo(0)
			end
			ClearCursor()
		end)
		
		-- BUTTON UPDATE
		
		nbBtn:SetScript("OnUpdate", function()
			nbUpdateButtonState(false)
		end)
		
		-- BUTTON RIGHT CLICK HANDLING
		
		nbBtn:SetScript("OnMouseUp", function(self, button)
			if button == "RightButton" then
				if nbBtn:IsMouseOver() then
					if IsShiftKeyDown() then
						svSilent = not svSilent
						nbUpdateTooltip()
					else
						svActiveFood = nil
						nbUpdateButtonState(true)
						nbUpdateTooltip()
					end
				end
				nbPlaySound("Sound/Interface/iUiInterfaceButtonA.ogg","SFX")
			end
		end)
		
		-- TOOLTIP
		
		nbBtn:SetScript("OnEnter", function()
			nbUpdateTooltip()
			GameTooltip:Show()
		end)
		
		nbBtn:SetScript("OnLeave", function()
			GameTooltip:Hide()
		end)
		
		nbLoaded = true
	end
end

-- Called at startup to verify player's class and level. Hides button
-- if it's not needed.
local function nbCheckPlayerClass()
	local _,_,notes = GetAddOnInfo("NoseBag")
	print(nbNameFormatted .. ": " .. notes)
	
	-- If we're not on a hunter, don't bother. If this is a hunter below
	-- level 10 then just hide the button for now.
	local _,_,classIndex = UnitClass("player")

	if classIndex == 3 then
		nbInitialize()
		C_Timer.NewTicker(nbHappinessCheckSeconds, nbCheckPetHappiness)
		
		nbPlayerLevel = UnitLevel("player")
		if nbPlayerLevel > 9 then
			nbShowButton()
		else
			print(nbNameFormatted .. ": Below level 10. Hiding button.")
			nbHideButton()
		end
	else
		nbHideButton()
	end
end

-- MAIN EVENT HANDLER

nbBtn:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_LOGIN" then
		C_Timer.After(1, nbCheckPlayerClass)
	elseif event == "PLAYER_REGEN_DISABLED" then
		nbInCombat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
		nbInCombat = false
	end
end)
